#!/bin/bash
set -e

# 创建保存DEB包的目录
DOWNLOAD_DIR="/tmp/test-p2p"
LOG_FILE="$DOWNLOAD_DIR/download_log.txt"

# 下载包的数量
TEST_PACK_NUM=10000

# 每次下载的最大数量
BATCH_SIZE=30

# 记录下载成功的包名的文件
SUCCESS_LOG="$DOWNLOAD_DIR/downloaded_packages.log"

# 记录下载失败的包名的文件
FAILURE_LOG="$DOWNLOAD_DIR/failed_packages.log"

# 记录下载失败的计数器
FAILURE_COUNT=0

# 限制的包大小，单位为字节
SIZE_LIMIT=200000000  # 200MB

# 检查是否是root权限
if [ "$EUID" -ne 0 ]; then
    echo "请以root权限运行此脚本。" | tee -a "$LOG_FILE"
    exit 1
fi

# 删除并重新创建下载目录
if [ -d "$DOWNLOAD_DIR" ]; then
    rm -rf "$DOWNLOAD_DIR"
fi
mkdir -p "$DOWNLOAD_DIR"

# 清空成功下载包名记录文件和失败记录文件
> "$SUCCESS_LOG"
> "$FAILURE_LOG"

# 将所有输出重定向到日志文件
exec > >(tee -a "$LOG_FILE") 2>&1

cd "$DOWNLOAD_DIR" || exit

apt update

# 获取所有可用的包
PACKAGES=$(apt list --all-versions 2>/dev/null | grep -oP '^[^\s/]+' | sort | uniq)

# 检查是否安装了shuf命令
if ! command -v shuf &> /dev/null; then
    echo "请安装 coreutils 包以使用 shuf 命令。" | tee -a "$LOG_FILE"
    exit 1
fi

# 随机选择指定数量的软件包
SELECTED_PACKAGES=$(echo "$PACKAGES" | shuf -n "$TEST_PACK_NUM")

# 初始化一个数组来存储符合条件的软件包
VALID_PACKAGES=()

# 验证每个选择的软件包是否存在且大小符合限制
echo "开始验证选定的软件包..."
for PACKAGE in $SELECTED_PACKAGES; do
    if apt-cache show "$PACKAGE" &> /dev/null; then
        # 获取软件包大小（单位：字节）
        PACKAGE_SIZE=$(apt-cache show "$PACKAGE" | grep '^Size:' | head -n 1 | awk '{print $2}')
        # 检查软件包大小是否小于限制的大小
        if [ "$PACKAGE_SIZE" -lt "$SIZE_LIMIT" ]; then
            VALID_PACKAGES+=("$PACKAGE")
        else
            echo "软件包 $PACKAGE 大小是$PACKAGE_SIZE, 超过限制，跳过下载。"
        fi
    else
        echo "软件包 $PACKAGE 不存在，跳过下载。"
    fi
done

# 分批下载符合条件的软件包
echo "开始分批下载符合条件的软件包..."
for (( i=0; i<${#VALID_PACKAGES[@]}; i+=BATCH_SIZE )); do
    BATCH=("${VALID_PACKAGES[@]:i:BATCH_SIZE}")
    echo "即将下载以下软件包："
    for PACKAGE in "${BATCH[@]}"; do
        echo "$PACKAGE"
    done

    if apt download "${BATCH[@]}"; then
        echo "${BATCH[@]}" >> "$SUCCESS_LOG"
    else
        for PACKAGE in "${BATCH[@]}"; do
            if ! ls | grep -q "${PACKAGE}_"; then
                echo "软件包 $PACKAGE 下载失败。"
                echo "$PACKAGE" >> "$FAILURE_LOG"
                (( FAILURE_COUNT++ ))
            fi
        done
    fi
done

echo "选定的软件包已下载到目录 $DOWNLOAD_DIR。"
echo "成功下载的软件包名单已保存到 $SUCCESS_LOG。"
echo "下载失败的软件包数量为: $FAILURE_COUNT"
echo "下载失败的软件包名单已保存到 $FAILURE_LOG。"
